package com.energy.manage.service.service.analysis.impl;

import com.alibaba.fastjson.JSONObject;
import com.energy.manage.common.reponse.ConstVar;
import com.energy.manage.common.reponse.ResultResp;
import com.energy.manage.common.util.HttpUtils;
import com.energy.manage.service.constant.analysis.AnalysisConstants;
import com.energy.manage.service.domain.dto.analysis.AnalysisDto;
import com.energy.manage.service.domain.dto.analysis.AnalysisResultDto;
import com.energy.manage.service.domain.dto.analysisalgorithm.DataContractDto;
import com.energy.manage.service.domain.dto.analysisalgorithm.DataContractTypeDto;
import com.energy.manage.service.domain.vo.analysis.*;
import com.energy.manage.service.domain.vo.windenginegroup.WindEngineGroupVo;
import com.energy.manage.service.mappers.analysis.*;
import com.energy.manage.service.mappers.windenginegroup.WindEngineGroupMapper;
import com.energy.manage.service.property.analysis.AlgorithmProperties;
import com.energy.manage.service.service.analysis.AnalysisService;
import com.energy.manage.service.util.BeanMapUtils;
import com.energy.manage.service.util.TreeUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 分析服务
 * @author chy
 * @date 2024/5/23 12:59
 * @desc
 */
@Slf4j
@Service
public class AnalysisServiceImpl implements AnalysisService {


    @Autowired
    private AnalysisResultMapper analysisResultMapper;

    @Autowired
    private AnalysisTypeMapper analysisTypeMapper;

    @Autowired
    private WindEngineGroupMapper engineGroupMapper;

    @Autowired
    private AnalysisCommentDescriptionMapper commentDescriptionMapper;

    @Autowired
    private AnalysisCommentTypeRelationMapper commentTypeRelationMapper;

    @Autowired
    private AnalysisDiagramRelationMapper diagramRelationMapper;

    @Autowired
    private AnalysisGeneralFileMapper generalFileMapper;

    /**
     * 分析结果列表
     * @param analysisResultDto
     * @return
     */
    @Override
    public List<AnalysisResultVo> analysisResultList(AnalysisResultDto analysisResultDto) {
        return analysisResultMapper.selectByCondition(analysisResultDto);
    }

    /**
     * 分析
     * @param analysisDto
     * @return
     */
    @Override
    public ResultResp analysis(AnalysisDto analysisDto) {
        try {
            String batchCode = analysisDto.getDataBatchNum();
            if(checkAnalysising(batchCode)){
                return ResultResp.FAIL(ConstVar.ANALYSISING);
            }
            //将配置文件中的算法json转换为对象
            ObjectMapper mapper = new ObjectMapper();
            JsonNode rootNode = mapper.readTree(AlgorithmProperties.analysisJson);
            //将前端入参转换为算法标准json
            dataProccess(analysisDto,rootNode);
            //调用算法
            JSONObject jsonObject = HttpUtils.doPost(AlgorithmProperties.analysisUrl,mapper.convertValue(rootNode, Map.class));
            log.info("调用算法分析返回结果：{}",jsonObject);
            return ResultResp.SUCCESS();
        } catch (JsonProcessingException e) {
            log.error("批次号 = {} 手动分析异常",analysisDto.getDataBatchNum(),e);
            return ResultResp.FAIL();
        }
    }

    /**
     * 分析编辑查询（查询分析类型、风机组）
     * @param batchCode
     * @return
     */
    @Override
    public AnalysisResultShowVo analysisEditQuery(String batchCode){
        AnalysisResultShowVo analysisResultShowVo = new AnalysisResultShowVo();
        return analysisResultShowVo.setAnalysisTypeList(getAnalysisType())
                .setWindEngineGroupList(getWindEngineGroupByBatchCode(batchCode));
    }


    /**
     * 分析详情
     * @param batchCode
     * @param analysisTypeCode
     * @param fieldEngineCode
     * @return
     */
    @Override
    public List<AnalysisDetailVo> analysisDetail(String batchCode, String analysisTypeCode, String fieldEngineCode) {
        List<AnalysisDetailVo> analysisDetailVos = new ArrayList<>();
        //获取所有的分析类型
        List<AnalysisTypeVo> analysisTypeVos = analysisTypeMapper.selectByTypeCode(analysisTypeCode);
        if(CollectionUtils.isEmpty(analysisTypeVos)){
            return analysisDetailVos;
        }
        //最新总文件
        List<AnalysisGeneralFileVo> newGeneralFileVos = new ArrayList<>();
        //如果风机编号为空的话 则需要查总文件
        if(StringUtils.isBlank(fieldEngineCode)){
            //获取最新总文件列表
            newGeneralFileVos = getNewGeneralFiles(batchCode,analysisTypeCode);
        }
        //获取最新分文件
        List<AnalysisDiagramRelationVo> newDiagramRelationVos = getNewDiagramRelations(batchCode, analysisTypeCode, fieldEngineCode);
        //查询评论
        List<AnalysisCommentDescriptionVo> commentDescriptionVos = commentDescriptionMapper.selecByBatchCodeAndAnalysisTypeCode(batchCode,analysisTypeCode);
        //根据分析类型设置分析结果
        List<AnalysisGeneralFileVo> finalNewGeneralFileVos = newGeneralFileVos;
        analysisTypeVos.forEach(analysisTypeVo -> {
            //设置相应参数
            AnalysisDetailVo analysisDetailVo = getAnalysisDetailVo(analysisTypeVo, finalNewGeneralFileVos,newDiagramRelationVos,commentDescriptionVos);
            if(analysisDetailVo != null){
                analysisDetailVos.add(analysisDetailVo);
            }
        });
        //只有当分析类型不为空的时候才返回评论类型
        if(StringUtils.isNotBlank(analysisTypeCode) && CollectionUtils.isNotEmpty(analysisDetailVos)){
            analysisDetailVos.get(0).setCommentTypeRelations(commentTypeRelationMapper.selectByAnalysisTypeCode(batchCode,analysisTypeCode));
        }
        return analysisDetailVos;
    }


    /**
     * 获取分析类型
     */
    private List<Map<String,Object>> getAnalysisType(){
        List<Map<String,Object>> resultList = new ArrayList<>();
        List<AnalysisTypeVo> analysisTypeVos =  analysisTypeMapper.selectAllVo();
        if(CollectionUtils.isEmpty(analysisTypeVos)){
            return resultList;
        }
        analysisTypeVos.forEach(analysisTypeVo -> resultList.add(BeanMapUtils.beanToMap(analysisTypeVo)));
        return TreeUtil.getJava8ResultTree(resultList, AnalysisConstants.ANALYSIS_TYPE_PARRENT_KEY);
    }

    /**
     * 获取风机组编号
     * @param batchCode
     * @return
     */
    private List<String> getWindEngineGroupByBatchCode(String batchCode){
        List<WindEngineGroupVo> engineGroupVos = engineGroupMapper.selectWindEngineGroupByBatchCode(batchCode);
        return engineGroupVos.parallelStream().map(WindEngineGroupVo::getEngineCode).collect(Collectors.toList());
    }


    /**
     * 获取总文件
     * @param batchCode
     * @param analysisTypeCode
     * @return
     */
    private List<AnalysisGeneralFileVo> getNewGeneralFiles(String batchCode,String analysisTypeCode){
        List<AnalysisGeneralFileVo> generalFileVos = generalFileMapper.selectByCondition(batchCode,analysisTypeCode);
        generalFileVos.sort(Comparator.comparing(AnalysisGeneralFileVo::getCreateTime,Comparator.reverseOrder()));
        //删除地址相同的文件
        return generalFileVos.stream().collect(Collectors.collectingAndThen(
                Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(AnalysisGeneralFileVo::getFileAddr))),
                ArrayList :: new));
    }


    /**
     * 获取最新分析结果分文件
     * @param batchCode
     * @param analysisTypeCode
     * @param fieldEngineCode
     * @return
     */
    private List<AnalysisDiagramRelationVo> getNewDiagramRelations(String batchCode,String analysisTypeCode,String fieldEngineCode){
        List<AnalysisDiagramRelationVo> diagramRelationVos = diagramRelationMapper.selectByCondition(batchCode,analysisTypeCode,fieldEngineCode);
        if(CollectionUtils.isEmpty(diagramRelationVos)){
            return diagramRelationVos;
        }
        //如果风机编号不为空的话则不对风机编号进行排序
        if (StringUtils.isNotBlank(fieldEngineCode)) {
            diagramRelationVos.sort(Comparator.comparing(AnalysisDiagramRelationVo::getCreateTime, Comparator.reverseOrder()));
        } else {
            diagramRelationVos.sort(Comparator.comparing(AnalysisDiagramRelationVo::getFieldEngineCode)
                    .thenComparing(AnalysisDiagramRelationVo::getCreateTime, Comparator.reverseOrder()));
        }
        //去除地址相同的对象
        return diagramRelationVos.stream().collect(Collectors.collectingAndThen(
                Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(AnalysisDiagramRelationVo::getFileAddr))),
                                                     ArrayList :: new)
        );
    }

    /**
     * 设置分析结果
     * @param analysisTypeVo
     * @param newGeneralFileVos
     * @param newDiagramRelationVos
     * @param commentDescriptionVos
     * @return
     */
    private AnalysisDetailVo getAnalysisDetailVo(AnalysisTypeVo analysisTypeVo,
                                                 List<AnalysisGeneralFileVo> newGeneralFileVos,
                                                 List<AnalysisDiagramRelationVo> newDiagramRelationVos,
                                                 List<AnalysisCommentDescriptionVo> commentDescriptionVos){
        List<AnalysisGeneralFileVo> generalFileVos = null;
        if(CollectionUtils.isNotEmpty(newGeneralFileVos)) {
            generalFileVos = newGeneralFileVos.stream().filter(analysisGeneralFileVo -> analysisGeneralFileVo.getAnalysisTypeCode().equals(analysisTypeVo.getTypeCode())).collect(Collectors.toList());
        }
        List<AnalysisDiagramRelationVo> diagramRelationVos = null;
        if(CollectionUtils.isNotEmpty(newDiagramRelationVos)){
            diagramRelationVos = newDiagramRelationVos.stream().filter(analysisDiagramRelationVo -> analysisDiagramRelationVo.getAnalysisTypeCode().equals(analysisTypeVo.getTypeCode())).collect(Collectors.toList());
        }
        if(CollectionUtils.isEmpty(generalFileVos) && CollectionUtils.isEmpty(diagramRelationVos)){
            return null;
        }
        return new AnalysisDetailVo()
                .setAnalysisTypeCode(analysisTypeVo.getTypeCode())
                .setAnalysisTypeName(analysisTypeVo.getTypeName())
                .setGeneralFiles(generalFileVos)
                .setDiagramRelations(diagramRelationVos)
                .setCommentDescriptionVos(commentDescriptionVos);
    }

    /**
     * 分析中检查
     * @param batchCode
     * @return
     */
    private boolean checkAnalysising(String batchCode){
        int analysisingCount = analysisResultMapper.selectAlasisingCountByBatchCode(batchCode);
        return analysisingCount > AnalysisConstants.SERVER_MERCHINE_COUNT;
    }

    /**
     * 数据处理
     * @param analysisDto 前端入参
     * @param rootNode 根节点json
     */
    private void dataProccess(AnalysisDto analysisDto,JsonNode rootNode){
        //前端入参类
        Field[] analysisFields = analysisDto.getClass().getDeclaredFields();
        Arrays.asList(analysisFields).stream().forEach(field -> {
            field.setAccessible(true);
            try {
                if(field.get(analysisDto) != null){
                    updateNodesByPropertyName(rootNode, field.getName(), field.get(analysisDto).toString());
                }
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        });
    }

    /**
     * 测试排序去重
     * @param args
     * @throws InterruptedException
     */
    public static void main(String[] args) throws InterruptedException {
        List<AnalysisGeneralFileVo> generalFileVos = new ArrayList<>();
        AnalysisGeneralFileVo a1 = new AnalysisGeneralFileVo();
        a1.setFileAddr("http://123/412/89.html");
        a1.setCreateTime(new Date());
        a1.setBatchCode("123");
        generalFileVos.add(a1);
        Thread.sleep(1000);
        AnalysisGeneralFileVo a2 = new AnalysisGeneralFileVo();
        a2.setFileAddr("http://123/412/89.html");
        a2.setCreateTime(new Date());
        generalFileVos.add(a2);
        a2.setBatchCode("453");
        generalFileVos.sort(Comparator.comparing(AnalysisGeneralFileVo::getCreateTime,Comparator.reverseOrder()));
        //删除地址相同的文件
        List<AnalysisGeneralFileVo> returnList =  generalFileVos.stream().collect(Collectors.collectingAndThen(
                                                    Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(AnalysisGeneralFileVo::getFileAddr))),
                                                    ArrayList :: new));
        returnList.forEach(System.out::println);
    }

    /**
     * 根据属性修改json值
     * @param node 节点
     * @param propertyName 属性名称
     * @param newValue 新值
     */
    private static void updateNodesByPropertyName(JsonNode node, String propertyName, String newValue) {
        if (node.isObject()) {
            node.fields().forEachRemaining(entry -> {
                // 匹配到属性名称，进行值的修改
                if (entry.getKey().equals(propertyName)) {
                    assignmentByPropertyName(node, propertyName, newValue);
                }
                // 递归搜索子节点
                updateNodesByPropertyName(entry.getValue(), propertyName, newValue);
            });
        }
    }

    /**
     * 处理特殊属性
     * @param node
     * @param propertyName
     * @param newValue
     */
    private static void  assignmentByPropertyName(JsonNode node, String propertyName, String newValue){
        if(AlgorithmProperties.graphSetsProperties.contains(propertyName)){
            return ;
        }
        if(AlgorithmProperties.configAnalysisProperties.equals(propertyName)){
            return;
        }
        ((ObjectNode) node).put(propertyName, newValue);
    }

}
